package ru.dbotthepony.mc.otm.datagen.advancements

import net.minecraft.advancements.Advancement
import net.minecraft.advancements.AdvancementHolder
import net.minecraft.advancements.AdvancementRewards
import net.minecraft.advancements.AdvancementType
import net.minecraft.advancements.Criterion
import net.minecraft.advancements.DisplayInfo
import net.minecraft.advancements.critereon.ContextAwarePredicate
import net.minecraft.advancements.critereon.EntityPredicate
import net.minecraft.advancements.critereon.InventoryChangeTrigger
import net.minecraft.advancements.critereon.ItemPredicate
import net.minecraft.core.registries.Registries
import net.minecraft.network.chat.Component
import net.minecraft.resources.ResourceKey
import net.minecraft.resources.ResourceLocation
import net.minecraft.tags.TagKey
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.ItemLike
import ru.dbotthepony.mc.otm.core.TextComponent
import ru.dbotthepony.mc.otm.datagen.modLocation
import java.util.function.Consumer

fun AdvancementBuilder(): Advancement.Builder = Advancement.Builder.advancement()

fun Advancement.Builder.save(serializer: Consumer<AdvancementHolder>, name: ResourceLocation): AdvancementHolder {
	val built = build(name)
	serializer.accept(built)
	return built
}

fun advancementLoot(location: ResourceLocation): AdvancementRewards.Builder {
	return AdvancementRewards.Builder.loot(ResourceKey.create(Registries.LOOT_TABLE, location))
}

fun advancementLoot(location: String): AdvancementRewards.Builder {
	return advancementLoot(modLocation(location))
}

fun AdvancementRewards.Builder.addLootTable(location: ResourceLocation): AdvancementRewards.Builder {
	return addLootTable(ResourceKey.create(Registries.LOOT_TABLE, location))
}

fun DisplayInfo(
	itemStack: ItemStack,
	title: Component = TextComponent("undefined"),
	description: Component = TextComponent("undefined"),
	background: ResourceLocation? = null,
	frameType: AdvancementType = AdvancementType.TASK,
	showToast: Boolean = true,
	announceChat: Boolean = true,
	hidden: Boolean = false,
): DisplayInfo {
	return DisplayInfo(
		itemStack,
		title,
		description,
		background,
		frameType,
		showToast,
		announceChat,
		hidden
	)
}

// compiler thinks it is shadowed - it is not
@Suppress("EXTENSION_SHADOWED_BY_MEMBER")
fun Advancement.Builder.display(
	itemStack: ItemStack,
	title: Component = TextComponent("undefined"),
	description: Component = TextComponent("undefined"),
	background: ResourceLocation? = null,
	frameType: AdvancementType = AdvancementType.TASK,
	showToast: Boolean = true,
	announceChat: Boolean = true,
	hidden: Boolean = false,
) = display(itemStack, title, description, background, frameType, showToast, announceChat, hidden)

fun predicate(tag: TagKey<Item>): ItemPredicate {
	return ItemPredicate.Builder.item().of(tag).build()
}

fun criterion(tag: TagKey<Item>): Criterion<*> {
	return InventoryChangeTrigger.TriggerInstance.hasItems(predicate(tag))
}

fun predicate(item: ItemLike): ItemPredicate {
	return ItemPredicate.Builder.item().of(item).build()
}

fun criterion(item: ItemLike): Criterion<*> {
	return InventoryChangeTrigger.TriggerInstance.hasItems(predicate(item))
}

fun EntityPredicate.wrap(): ContextAwarePredicate = EntityPredicate.wrap(this)
